#=
Software for automated data analysis of Wilson_tm fermions.
Go to SET UP VARIABLES  section to properly configure before running.
You have to up set the following variables:
- path_data where data are stored
- path_plat where a plat.txt file is stored with plateaus for all ensembles
- path_result where you want to save results.txt and plots 
- ensembles to analyse 
- sector you are interested in: light-light, light-heavy, heavy-heavy...
- rwf set to true if you want to include reweighting factors
- compute_t0 set to true if you want to extract t0 from ms.dat file. Otherwise, t0 is taken from 1608.08900 in const.jl
- mass_shift set to true if you want to include the mass shift
- go to line 477 to modify continuum limit + chiral extrapolatio fit model

Data in path_data need to be stored as follow:
data->
    -ens_id_1->
             -mesons.dat
             -ms.dat
             -ms1.dat
             -pbp.dat
    -ens_id_2->
             -r0_mesons.dat
             -r1_mesons.dat
             -r0_ms.dat
             -r1_ms.dat
             -r0_ms1.dat
             -r1_ms1.dat
             -r0_pbp.dat
             -r1_pbp.dat
    -and so on
=#

using Revise, juobs, PyPlot, LaTeXStrings, ADerrors, DelimitedFiles

#============= SET UP VARIABLES ===========#
const path_data = "/media/alessandro/4277fef2-edc5-4e0d-89cb-f5d1d44fbc8c/data"
const path_plat = "/home/alessandro/automated-analysis/plat.txt"
const path_results = "/home/alessandro/Desktop/results"
const ensembles = ["J303", "H400", "N200" , "N202", "N300", "H101", "N203"]
const sector = Dict("ll"=>true, "ls"=>true, "lh"=>true, "ss"=>true, "sh"=>true, "hh"=>true )
const rwf = false
const compute_t0 = false
const mass_shift = false #not implemented yet
#go to line 443 to change CL+chiral extrapolation fit model

#========= DO NOT CHANGE AFTER THIS LINE ==========#
include("types.jl")
include("tools.jl")
include("const.jl")

#======== GET ENSEMBLE INFORMATION FROM DATABASE ==========#
ensinfo = Vector{EnsInfo}(undef, length(ensembles))
for i in 1:length(ensembles)
    ens = ensembles[i]
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end

#======== ANALYSIS ==========#

println("\n Computing Observables \n")

    
@time begin
    
    ensobs = Vector{EnsObs}(undef,length(ensembles))
    for i in 1:length(ensinfo)
        @time begin
            ens = ensinfo[i]
            println("\n Ensemble: ", ens.id)
            pp = get_corr(ens,"G5", "G5", rw=rwf)
            a1a1 = get_corr(ens, "G1G5", "G1G5", rw=rwf)
            mu_list = getfield.(pp, :mu)
            m_ps = get_meff(pp, ens)
            m_vec = get_meff(a1a1, ens)
            println("\n pseudo mass ", m_ps, "\n")
            println("\n vec mass ", m_vec, "\n")
            f_ps = get_f(pp, m_ps, ens)
            f_vec = get_f(a1a1, m_vec, ens)
            ensobs[i] =  EnsObs(ens, mu_list, m_ps, f_ps, m_vec, f_vec)
            if ens.deg
                ensobs[i].m_ls = ensobs[i].m_ll
                ensobs[i].m_ls_vec = ensobs[i].m_ll_vec
                ensobs[i].m_ss = ensobs[i].m_ll
                ensobs[i].m_ss_vec = ensobs[i].m_ll_vec
                ensobs[i].m_sh = ensobs[i].m_lh
                ensobs[i].m_sh_vec = ensobs[i].m_lh_vec
                ensobs[i].f_ls = ensobs[i].f_ll
                ensobs[i].f_ls_vec = ensobs[i].f_ll_vec
                ensobs[i].f_ss = ensobs[i].f_ll
                ensobs[i].f_ss_vec = ensobs[i].f_ll_vec
                ensobs[i].f_sh = ensobs[i].f_lh
                ensobs[i].f_sh_vec = ensobs[i].f_lh_vec
            end
            println(" Time:")
        end
    end
    println("Total time:")
end
    

#=========== MATCHING =============#

println("\n Matching Procedure \n")

@time begin
    for i in 1:length(ensobs)
        ens = ensobs[i]
        mul, mus, muh = get_mu(ens.mu_list, ens.ensinfo.deg)
        if compute_t0
            est_t0 = get_t0(ens.ensinfo, rw=rwf)
            ens.t0 = est_t0
            est_a = t0_ph[1] / sqrt(8 * ens.t0)
            ens.a = est_a
        else
            ens.t0 = t0(ens.ensinfo.beta)
            ens.a = a(ens.ensinfo.beta)
        end
        target = ens.a * (2/3*M[1] + 1/3* M[3])/ hc
        if !ens.ensinfo.deg
            ens.muh_target = match_muc(muh, ens.m_lh, ens.m_sh, target)
        else
            ens.muh_target = match_muc(muh, ens.m_lh, ens.m_sh, target)
        end
        uwerr(ens.muh_target)
        println(t0, "\n", a, "\n", ens.muh_target)
        
        # interpolate lh
        if sector["lh"]
            # m_lh
            par, chi2exp = lin_fit(muh, ens.m_lh)
            ens.m_lh_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.m_lh_match)
            #m_lh_star
            par, chi2exp = lin_fit(muh, ens.m_lh_vec)
            ens.m_lh_vec_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.m_lh_vec_match)
            # f_lh
            par, chi2exp = lin_fit(muh, ens.f_lh)
            ens.f_lh_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.f_lh_match)
            #f_lh_star
            par, chi2exp = lin_fit(muh, ens.f_lh_vec)
            ens.f_lh_vec_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.f_lh_vec_match)
        end
        # interpolate sh
        if sector["sh"]
            if ens.ensinfo.deg
                ens.m_sh_match = ens.m_lh_match
                ens.m_sh_vec_match = ens.m_lh_vec_match
                ens.f_sh_match = ens.f_lh_match
                ens.f_sh_vec_match = ens.f_lh_vec_match
            else
                # m_sh
                par, chi2exp = lin_fit(muh, ens.m_sh)     
                ens.m_sh_match = y_lin_fit(par, value(ens.muh_target))
                uwerr(ens.m_sh_match)
                # m_sh_star
                par, chi2exp = lin_fit(muh, ens.m_sh_vec)     
                ens.m_sh_vec_match = y_lin_fit(par, value(ens.muh_target))
                uwerr(ens.m_sh_vec_match)
                # f_sh
                par, chi2exp = lin_fit(muh, ens.f_sh)     
                ens.f_sh_match = y_lin_fit(par, value(ens.muh_target))
                uwerr(ens.f_sh_match)
                # f_sh_star
                par, chi2exp = lin_fit(muh, ens.f_sh_vec)     
                ens.f_sh_vec_match = y_lin_fit(par, value(ens.muh_target))
                uwerr(ens.f_sh_vec_match)
            end
        end
        # interpolate hh
        if sector["hh"]
            # m_hh
            par, chi2exp = lin_fit(muh, ens.m_hh)
            ens.m_hh_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.m_hh_match)
            # m_hh_vec
            par, chi2exp = lin_fit(muh, ens.m_hh_vec)
            ens.m_hh_vec_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.m_hh_vec_match)
            # f_hh
            par, chi2exp = lin_fit(muh, ens.f_hh)
            ens.f_hh_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.f_hh_match)
            # f_hh_vec
            par, chi2exp = lin_fit(muh, ens.f_hh_vec)
            ens.f_hh_vec_match = y_lin_fit(par, value(ens.muh_target))
            uwerr(ens.f_hh_vec_match)
        end
    end
    println(" Total time:")
end
    
    
#========== MATCHING PLOTS ==========#

println("\n Matchin Plots \n ")
check_folder = filter(x-> occursin("plots", x), readdir(path_results))
if !("plots" in check_folder)
    mkdir(joinpath(path_results,"plots"))
end
path_plot = joinpath(path_results,"plots")
for i in 1:length(ensobs)
    ens = ensobs[i]
    mul, mus, muh = get_mu(ens.mu_list, ens.ensinfo.deg)
    if sector["lh"]
        # masses
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$aM$")
        #m_lh
        errorbar(muh, value.(ens.m_lh), yerr=err.(ens.m_lh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_lh_match), yerr=err(ens.m_lh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #m_lh_star
        errorbar(muh, value.(ens.m_lh_vec), yerr=err.(ens.m_lh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_lh_vec_match), yerr=err(ens.m_lh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$m_D$", L"$\langle m_D \rangle$", L"$m_{D^*}$", L"$ \langle m_{D^*}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_mD_mDvec.pdf")
        savefig(joinpath(path_plot, name))
        close()
        
        # decays
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$af$")
        #m_lh
        errorbar(muh, value.(ens.f_lh), yerr=err.(ens.f_lh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_lh_match), yerr=err(ens.f_lh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #m_lh_star
        errorbar(muh, value.(ens.f_lh_vec), yerr=err.(ens.f_lh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_lh_vec_match), yerr=err(ens.f_lh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$f_D$", L"$\langle f_D \rangle$", L"$f_{D^*}$", L"$ \langle f_{D^*}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_fD_fDvec.pdf")
        savefig(joinpath(path_plot, name))
        close()
    end
    if sector["sh"] && !ens.ensinfo.deg
        # masses
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$aM$")
        #m_sh
        errorbar(muh, value.(ens.m_sh), yerr=err.(ens.m_sh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_sh_match), yerr=err(ens.m_sh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #m_sh_star
        errorbar(muh, value.(ens.m_sh_vec), yerr=err.(ens.m_sh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_sh_vec_match), yerr=err(ens.m_sh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$m_{D_s}$", L"$\langle m_{D_s} \rangle$", L"$m_{D^*_s}$", L"$ \langle m_{D^*_s}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_mDs_mDsvec.pdf")
        savefig(joinpath(path_plot, name))
        close()
        
        # decays
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$af$")
        #f_sh
        errorbar(muh, value.(ens.f_sh), yerr=err.(ens.f_sh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_sh_match), yerr=err(ens.f_sh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #fsh_star
        errorbar(muh, value.(ens.f_sh_vec), yerr=err.(ens.f_sh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_sh_vec_match), yerr=err(ens.f_sh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$f_{D_s}$", L"$\langle f_{D_s} \rangle$", L"$f_{D^*_s}$", L"$ \langle f_{D^*_s}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_fDs_fDsvec.pdf")
        savefig(joinpath(path_plot, name))
        close()
    end
    if sector["hh"] 
        # masses
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$aM$")
        #m_hh
        errorbar(muh, value.(ens.m_hh), yerr=err.(ens.m_hh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_hh_match), yerr=err(ens.m_hh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #m_hh_star
        errorbar(muh, value.(ens.m_hh_vec), yerr=err.(ens.m_hh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.m_hh_vec_match), yerr=err(ens.m_hh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$m_{\eta_c}$", L"$\langle m_{\eta_c} \rangle$", L"$m_{J/\psi}$", L"$ \langle m_{J/\psi}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_metac_mJpsi.pdf")
        savefig(joinpath(path_plot, name))
        close()
        
        # decays
        figure()
        title(ens.ensinfo.id)
        xlabel(L"$a\mu$")
        ylabel(L"$af$")
        #f_hh
        errorbar(muh, value.(ens.f_hh), yerr=err.(ens.f_hh), fmt="*", ms=5, mfc="none", capsize=2, c="tomato", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_hh_match), yerr=err(ens.f_hh_match), fmt="*", ms=5, mfc="none", capsize=2, c="darkred", lw=1)
        #f_hh_star
        errorbar(muh, value.(ens.f_hh_vec), yerr=err.(ens.f_hh_vec), fmt="*", ms=5, mfc="none", capsize=2, c="cornflowerblue", lw=1)
        errorbar(value(ens.muh_target), xerr=err(ens.muh_target), value(ens.f_hh_vec_match), yerr=err(ens.f_hh_vec_match), fmt="*", ms=5, mfc="none", capsize=2, c="navy", lw=1)
        legend([L"$f_{\eta_c}$", L"$\langle f_{\eta_c} \rangle$", L"$f_{J/\psi}$", L"$ \langle f_{J/\psi}\rangle $"])  
        display(gcf())
        name = string(ens.ensinfo.id,"_match_fetac_fJpsi.pdf")
        savefig(joinpath(path_plot, name))
        close()
    end
end

#======= RESULTS ==========#
# charm quark mass 
muc = zm_tm.(getfield.(ensinfo,:beta)) .* getfield.(ensobs,:muh_target) .* sqrt.(8 * getfield.(ensobs,:t0))
uwerr.(muc)

# ll
m_pi = Vector{uwreal}(undef, length(ensembles))
m_rho =Vector{uwreal}(undef, length(ensembles))
f_pi = Vector{uwreal}(undef, length(ensembles))
f_rho =Vector{uwreal}(undef, length(ensembles))
if sector["ll"]
    try 
        m_pi .= getfield.(ensobs, :m_ll) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_rho .= getfield.(ensobs, :m_ll_vec) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_pi .= getfield.(ensobs, :f_ll) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_rho .= getfield.(ensobs, :f_ll_vec) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_pi)
        uwerr.(m_rho)
        uwerr.(f_pi)
        uwerr.(f_rho)
    catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a light-light sector.")
    end
end
# ls
m_k = Vector{uwreal}(undef, length(ensembles))
m_k_star =Vector{uwreal}(undef, length(ensembles))
f_k= Vector{uwreal}(undef, length(ensembles))
f_k_star =Vector{uwreal}(undef, length(ensembles))
if sector["ls"] 
    try 
        m_k .= getfield.(ensobs, :m_ls) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_k_star .= getfield.(ensobs, :m_ls_vec) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_k .= getfield.(ensobs, :f_ls) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_k_star .= getfield.(ensobs, :f_ls_vec) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_k)
        uwerr.(m_k_star)
        uwerr.(f_k)
        uwerr.(f_k_star)
    catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a light-stange sector.")
    end
end
# lh
m_D = Vector{uwreal}(undef, length(ensembles))
m_D_star =Vector{uwreal}(undef, length(ensembles))
f_D= Vector{uwreal}(undef, length(ensembles))
f_D_star =Vector{uwreal}(undef, length(ensembles))
if sector["lh"] 
    try 
        m_D .= getfield.(ensobs, :m_lh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_D_star .= getfield.(ensobs, :m_lh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_D .= getfield.(ensobs, :f_lh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_D_star .= getfield.(ensobs, :f_lh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_D)
        uwerr.(m_D_star)
        uwerr.(f_D)
        uwerr.(f_D_star)
catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a light-heavy sector.")
    end
end
# ss
m_etaprime = Vector{uwreal}(undef, length(ensembles))
m_phi =Vector{uwreal}(undef, length(ensembles))
f_etaprime= Vector{uwreal}(undef, length(ensembles))
f_phi =Vector{uwreal}(undef, length(ensembles))
if sector["ss"] 
    try 
        m_etaprime .= getfield.(ensobs, :m_ss) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_phi .= getfield.(ensobs, :m_ss_vec) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_etaprime .= getfield.(ensobs, :f_ss) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_phi .= getfield.(ensobs, :f_ss_vec) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_etaprime)
        uwerr.(m_phi)
        uwerr.(f_etaprime)
        uwerr.(f_phi)
catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a strange-strange sector.")
    end
end
# sh 
m_Ds = Vector{uwreal}(undef, length(ensembles))
m_Ds_star =Vector{uwreal}(undef, length(ensembles))
f_Ds= Vector{uwreal}(undef, length(ensembles))
f_Ds_star =Vector{uwreal}(undef, length(ensembles))
if sector["sh"] 
    try 
        m_Ds .= getfield.(ensobs, :m_sh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_Ds_star .= getfield.(ensobs, :m_sh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_Ds .= getfield.(ensobs, :f_sh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_Ds_star .= getfield.(ensobs, :f_sh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_Ds)
        uwerr.(m_Ds_star)
        uwerr.(f_Ds)
        uwerr.(f_Ds_star)
catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a strange-heavy sector.")
    end
end
# hh 
m_etac = Vector{uwreal}(undef, length(ensembles))
m_jpsi =Vector{uwreal}(undef, length(ensembles))
f_etac= Vector{uwreal}(undef, length(ensembles))
f_jpsi =Vector{uwreal}(undef, length(ensembles))
if sector["hh"] 
    try 
        m_etac .= getfield.(ensobs, :m_hh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        m_jpsi .= getfield.(ensobs, :m_hh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_etac .= getfield.(ensobs, :f_hh_match) .* sqrt.(8 * getfield.(ensobs,:t0))
        f_jpsi .= getfield.(ensobs, :f_hh_vec_match) .* sqrt.(8 * getfield.(ensobs,:t0))        
        uwerr.(m_etac)
        uwerr.(m_jpsi)
        uwerr.(f_etac)
        uwerr.(f_jpsi)
catch
        error("Check ensobs[i].mu_list to see whether all the ensembles have a heavy-heavy sector.")
    end
end

#===== CONTINUUM AND CHIRAL EXTRAPOLATION =======#
##
println("\n Continuum and Chiral Extrapolation \n")
obs_db =Dict(
    "muc"=> [muc],
    "ll" => [m_pi, m_rho, f_pi, f_rho],
    "ls" => [m_k, m_k_star, f_k, f_k_star],
    "lh" => [m_D, m_D_star, f_D, f_D_star],
    "ss" => [m_etaprime, m_phi, f_etaprime, f_phi],
    "sh" => [m_Ds, m_Ds_star, f_Ds, f_Ds_star],
    "hh" => [m_etac, m_jpsi, f_etac, f_jpsi]
)


#preparing observables of interest (data, name, ylabel)
obs = Vector{Vector{uwreal}}(undef,0)
ttl_obs = Vector{String}(undef,0)
ylbl = Vector{String}(undef,0)
push!(obs, obs_db["muc"][1])
push!(ttl_obs, ttl_obs_db["muc"][1])
push!(ylbl, ylbl_db["muc"][1])

label = findall(sector)
for lab in label
    for i in 1:length(obs_db[lab])
        push!(obs, obs_db[lab][i])
        push!(ttl_obs, ttl_obs_db[lab][i])
        push!(ylbl, ylbl_db[lab][i])
    end
end
# estimate phi2 from database or form analysis if ll sector is computed 
phi2 = Vector{uwreal}(undef,length(ensembles))
sector["ll"] ? phi2 = 8 .* getfield.(ensobs,:t0) .* getfield.(ensobs,:m_ll).^2 : phi2 = 8 .* getfield.(ensobs,:t0) .* getfield.(getfield.(ensobs,:ensinfo),:mpi).^2
phi2_ph = (t0_ph[1] * 139.57039 / hc)^2
uwerr(phi2_ph)
#fit routine 
x = [1 ./(8 .* getfield.(ensobs,:t0)) phi2] #x[1] = a^2 / (8t0), x[2] = 8t0 mpi^2
uwerr.(x)
#modify CL+chiral fit  model
@. cl_chir_model(x, p) = (p[1] + p[2] * x[:, 1]) + (p[3]) * x[:, 2] #linear fits

obs_t0 = Vector{uwreal}(undef, length(obs)) #sqrt(8t0)obs @ CL & phi2_phys
xarr = Float64.(range(0.0, stop=0.05, length=100))
for k = 1:length(obs)
    par = fit_routine(cl_chir_model, value.(x), obs[k], 4)
    obs_t0[k] = par[1] + par[3] * phi2_ph
    uwerr(obs_t0[k])    
    y = Vector{uwreal}(undef, 100)
    for j=1:100
        y[j] = par[1] + par[2] * xarr[j] + par[3] * phi2_ph
    end
    uwerr.(y)
    figure()
    for b in unique(phi2) #select point with same beta
        nn = findall(x-> x == b, phi2)
        lgnd = string(L"$m_\pi = $", Int32(round(sqrt(value(b))*hc/t0_ph_value[1], sigdigits=3)), " MeV")
        errorbar(value.(x[nn,1]), value.(obs[k][nn]), err.(obs[k][nn]), ms=5, fmt="s", mfc="none", mew=0.8, elinewidth=0.8, capsize=2, label=lgnd)
    end
    fill_between(xarr, value.(y) .+ err.(y),value.(y) .- err.(y) , color="tomato", alpha=0.4)
    lgnd=L"$\mathrm{CL}$"
    errorbar(0, value(obs_t0[k]), err(obs_t0[k]), fmt="s", zorder=2, ms=4,  mfc="red", c = "red", mew=0.8, elinewidth=1, capsize=2, label=lgnd)
    axvline(0, ls="--", color="black", zorder=1, lw=0.6, label="")
    xlabel(L"$a^2/8t_0$")
    ylabel(ylbl[k])
    xlim(-0.002,0.05)
    #ylim(2.8,3.4)
    legend(ncol=2)
    display(gcf())
    t = string("fit_", ttl_obs[k], ".pdf")
    savefig(joinpath(path_plot, t))
    #savefig(joinpath("/Users/ale/Desktop/plottest", t))
    close()

end

obs_ph = Vector{uwreal}(undef, length(obs))
for k = 1:length(obs)
    println(ylbl[k], " = ", obs_t0[k])
    #phys
    obs_ph[k] = obs_t0[k] * hc / t0_ph[1]
    uwerr(obs_ph[k])
    println(ttl_obs[k], "(MeV) = ", obs_ph[k])
end

#======== STORE RESULTS ========#

open(joinpath(path_results, "results.txt"), "w") do f 
    print(f, "Data analysis results", "\n \n")
    print(f, "#================================# \n \n")
    print(f, "Setup:\n")
    print(f, "Ensembles analysed: ", ensembles, "\n")
    print(f, "Data storage: ", path_data, "\n")
    print(f, "Plateaux file: ", path_plat, "\n")
    print(f, "Results storage: ", path_results, "\n")
    print(f, "Sector analysed: ", findall(sector), "\n")
    print(f, "Reweighting factor: ", rwf, "\n")
    print(f, "Mass shift: ", mass_shift, "\n")
    if compute_t0
        print(f, "t0 values: computed from the ms.dat files \n")
    else
        print(f, "t0 values: extracted from const.jl \n")
    end
    if sector["ll"]
        print(f, "Phi2 = 8t0m_pi^2: m_pi computed from mesons.dat files \n")
    else
        print(f, "Phi2 = 8t0m_pi^2: m_pi extracted from ens_db in const.jl \n")
    end
    print(f, " \n#================================# \n \n")
    print(f, "\n Results at finite lattice spacings \n \n")

    lab = findall(sector)
    for i in 1:length(ensembles)
        print(f, "Ensemble ", ensobs[i].ensinfo.id, "\n")
        for j in 1:length(obs)
            print(f, ylbl[j], " = ", obs[j][i], "\n")
        end
        print(f, "\n")
    end

    print(f, " \n#================================# \n \n")
    print(f, "\n Results in the continuum limit \n \n")
    for k = 1:length(obs)
        print(f, ylbl[k], " = ", obs_t0[k], "\n")
        #phys
        print(f, ttl_obs[k], "(MeV) = ", obs_ph[k], "\n\n")
    end
end